@@ -315,7 +315,7 @@ binding_unregister_keys (GsdKeybindingsManager *manager)
 
         if (need_flush)
                 gdk_flush ();
-        gdk_error_trap_pop ();
+        gdk_error_trap_pop_ignored ();
 }
 
 static void
@@ -412,10 +412,10 @@ get_exec_environment (XEvent *xevent)
         int        i;
         int        display_index = -1;
         GdkScreen *screen = NULL;
-        GdkWindow *window = gdk_xid_table_lookup (xevent->xkey.root);
+        GdkWindow *window = gdk_x11_window_lookup_for_display (gdk_display_get_default (), xevent->xkey.root);
 
         if (window) {
-                screen = gdk_drawable_get_screen (GDK_DRAWABLE (window));
+                screen = gdk_window_get_screen (window);
         }
 
         g_return_val_if_fail (GDK_IS_SCREEN (screen), NULL);
@@ -630,20 +630,25 @@ gsd_keybindings_manager_stop (GsdKeybindingsManager *manager)
 
         binding_unregister_keys (manager);
 
-        g_slist_free (p->screens);
-        p->screens = NULL;
+        if (p->screens != NULL) {
+                g_slist_free (p->screens);
+                p->screens = NULL;
+        }
 
-        for (l = p->binding_list; l; l = l->next) {
-                Binding *b = l->data;
-                g_free (b->binding_str);
-                g_free (b->action);
-                g_free (b->gconf_key);
-                g_free (b->previous_key.keycodes);
-                g_free (b->key.keycodes);
-                g_free (b);
+        if (p->binding_list != NULL) {
+                for (l = p->binding_list; l; l = l->next) {
+                        Binding *b = l->data;
+                        g_free (b->binding_str);
+                        g_free (b->action);
+                        g_free (b->gconf_key);
+                        g_free (b->previous_key.keycodes);
+                        g_free (b->key.keycodes);
+                        g_free (b);
+                }
+
+                g_slist_free (p->binding_list);
+                p->binding_list = NULL;
         }
-        g_slist_free (p->binding_list);
-        p->binding_list = NULL;
 }
 
 static void
